package net.knarcraft.minecraftserverlauncher;

import net.knarcraft.minecraftserverlauncher.userinterface.GUI;

import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.util.Scanner;

/**
 * A holding class for methods shared between classes.
 *
 * @author      Kristian Knarvik <kristian.knarvik@knett.no>
 * @version     1.0.0
 * @since       1.0.0
 */
public class Shared {
    /**
     * Gets a resource as an InputStream
     * @param resourceName <p>The name of the resource you want to read.</p>
     * @return <p>An input stream which can be used to access the resource.</p>
     */
    public static InputStream getResourceAsStream(String resourceName) {
        ClassLoader classloader = Thread.currentThread().getContextClassLoader();
        return classloader.getResourceAsStream(resourceName);
    }

    /**
     * Gets a resource as a Scanner
     * @param resourceName <p>The name of the resource you want to read.</p>
     * @return <p>A scanner which can be used to read contents of the resource.</p>
     * @throws FileNotFoundException <p>If the resource is not found.</p>
     */
    public static Scanner getResourceAsScanner(String resourceName) throws FileNotFoundException {
        InputStream is = getResourceAsStream(resourceName);
        if (is == null) {
            throw new FileNotFoundException("The resource was not found.");
        }
        return new Scanner(is);
    }

    /**
     * Finds a substring between two substrings in a string.
     *
     * @param string The string containing the substrings
     * @param start  The substring before the wanted substring
     * @param end    The substring after the wanted substring
     * @return The wanted substring.
     */
    public static String stringBetween(String string, String start, String end) {
        int startPos = string.indexOf(start) + start.length();
        if (!string.contains(start) || string.indexOf(end, startPos) < startPos) {
            return "";
        }
        return string.substring(startPos, string.indexOf(end, startPos));
    }

    /**
     * Reads a file from a website.
     * This is used to find the newest version of jars and the software.
     *
     * @param path  The full url of the file to read
     * @return      True if successful. False otherwise
     */
    public static String readFile(String path) throws IOException {
        URL url = new URL(path);
        return new Scanner(url.openStream()).useDelimiter("\\Z").next();
    }

    /**
     * Downloads a file from a website.
     *
     * @param path      The full url of the file to download.
     * @param outfile   The file to save to
     * @return          True if successful. False otherwise
     */
    public static boolean downloadFile(String path, Path outfile) {
        try {
            URL url = new URL(path);
            InputStream in = url.openStream();
            Files.copy(in, outfile, StandardCopyOption.REPLACE_EXISTING);
            return true;
        } catch (IOException e) {
            return false;
        }
    }

    /**
     * Recursivly copies a folder to another location
     *
     * @param src <p>The folder to copy</p>
     * @param dest <p>Target destination</p>
     * @throws IOException <p>If we can't start a file stream</p>
     */
    public static void copyFolder(GUI gui, File src, File dest) throws IOException {
        if (!src.isDirectory()) {
            InputStream in = new FileInputStream(src);
            OutputStream out = new FileOutputStream(dest);
            byte[] buffer = new byte[1024];
            int length;
            while ((length = in.read(buffer)) > 0){
                out.write(buffer, 0, length);
            }
            in.close();
            out.close();
            gui.setStatus("Copied file " + src);
        } else {
            if(!dest.exists()){
                if (dest.mkdir()) {
                    gui.setStatus("Copied directory " + src);
                } else {
                    return;
                }
            }
            String[] files = src.list();
            if (files != null) {
                for (String file : files) {
                    File srcFile = new File(src, file);
                    File destFile = new File(dest, file);
                    copyFolder(gui, srcFile, destFile);
                }
            }
        }
    }

    /**
     * Opens an url in the user's default application.
     * @param url <p>URL to open</p>
     */
    public static void goToURL(String url) {
        java.awt.Desktop desktop = java.awt.Desktop.getDesktop();
        try {
            desktop.browse(new URI(url));
        } catch (URISyntaxException | IOException e1) {
            e1.printStackTrace();
        }
    }
}
